@@ -41,7 +41,9 @@ GeanyData	*geany_data;
 PLUGIN_VERSION_CHECK(224)
 
 PLUGIN_SET_TRANSLATABLE_INFO(LOCALEDIR, GETTEXT_PACKAGE,
-	_("Insert Numbers"), _("Insert/Fill columns with numbers."),
+	_("Insert Numbers"), _("Insert/Fill columns with numbers."
+	"\nThis plugin currently has no maintainer. Would you like to help"
+	" by contributing to this plugin?"),
 	"0.2.2", "Dimitar Toshkov Zhekov <dimitar.zhekov@gmail.com>")
 
 /* Keybinding(s) */
@@ -309,7 +311,7 @@ static void set_entry(GtkWidget *entry, gint maxlen, GtkWidget *label, const gch
 	gtk_entry_set_max_length(GTK_ENTRY(entry), maxlen);
 	gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
-	ui_widget_set_tooltip_text(entry, tooltip);
+	gtk_widget_set_tooltip_text(entry, tooltip);
 }
 
 #if !GTK_CHECK_VERSION(3, 0, 0)
@@ -376,20 +378,20 @@ static void on_insert_numbers_activate(G_GNUC_UNUSED GtkMenuItem *menuitem, G_GN
 	gtk_table_attach(grid, GTK_WIDGET(combo), 1, 3, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 #endif
 	label = gtk_label_new(_("Letters:"));
-	ui_widget_set_tooltip_text(label, case_tip);
+	gtk_widget_set_tooltip_text(label, case_tip);
 	gtk_grid_attach(grid, label, 3, 1, 1, 1);
 	upper = gtk_radio_button_new_with_mnemonic(NULL, _("_Upper"));
-	ui_widget_set_tooltip_text(upper, case_tip);
+	gtk_widget_set_tooltip_text(upper, case_tip);
 	gtk_grid_attach(grid, upper, 4, 1, 1, 1);
 	d.lower = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(upper));
-	ui_widget_set_tooltip_text(label, case_tip);
+	gtk_widget_set_tooltip_text(label, case_tip);
 	label = gtk_label_new_with_mnemonic(_("_Lower"));
-	ui_widget_set_tooltip_text(label, case_tip);
+	gtk_widget_set_tooltip_text(label, case_tip);
 	gtk_container_add(GTK_CONTAINER(d.lower), label);
 	gtk_grid_attach(grid, d.lower, 5, 1, 1, 1);
 
 	d.prefix = gtk_check_button_new_with_mnemonic(_("Base _prefix"));
-	ui_widget_set_tooltip_text(d.prefix,
+	gtk_widget_set_tooltip_text(d.prefix,
 		_("0 for octal, 0x for hex, + for positive decimal"));
 	gtk_grid_attach(grid, d.prefix, 1, 2, 2, 1);
 	label = gtk_label_new(_("Padding:"));
@@ -474,14 +476,14 @@ void plugin_init(G_GNUC_UNUSED GeanyData *data)
 	step_value = 1;
 	base_value = 10;
 
-	main_menu_item = gtk_menu_item_new_with_mnemonic(_("Insert _Numbers"));
+	main_menu_item = gtk_menu_item_new_with_mnemonic(_("Insert _Numbers..."));
 	gtk_widget_show(main_menu_item);
 	gtk_container_add(GTK_CONTAINER(geany->main_widgets->tools_menu), main_menu_item);
 	g_signal_connect(main_menu_item, "activate", G_CALLBACK(on_insert_numbers_activate),
 		NULL);
 
 	keybindings_set_item(plugin_key_group, INSERT_NUMBERS_KB, on_insert_numbers_key,
-		0, 0, "insert_numbers", _("Insert Numbers"), main_menu_item);
+		0, 0, "insert_numbers", _("Insert Numbers..."), main_menu_item);
 
 	plugin_signal_connect(geany_plugin, G_OBJECT(geany->main_widgets->tools_menu), "show",
 		FALSE, (GCallback) on_tools_show, NULL);
