@@ -48,6 +48,13 @@ typedef enum
 	MATCH_PATTERN
 } MatchType;
 
+typedef struct
+{
+	GeanyProject *project;
+	GPtrArray *expanded_paths;
+} ExpandData;
+
+
 static GdkColor s_external_color;
 static GtkWidget *s_toolbar = NULL;
 static gboolean s_pending_reload = FALSE;
@@ -876,7 +883,7 @@ static void create_branch(gint level, GSList *leaf_list, GtkTreeIter *parent,
 {
 	GSList *dir_list = NULL;
 	GSList *file_list = NULL;
-	GSList *elem;
+	GSList *elem = NULL;
 
 	foreach_slist (elem, leaf_list)
 	{
@@ -914,36 +921,13 @@ static void create_branch(gint level, GSList *leaf_list, GtkTreeIter *parent,
 			g_free(content_type);
 		}
 
-		if (patterns_match(header_patterns, path_arr[level]))
-		{
-			if (! icon)
-				icon = g_icon_new_for_string("prjorg-header", NULL);
+		if (! icon)
+			icon = g_themed_icon_new("text-x-generic");
 
-			gtk_tree_store_insert_with_values(s_file_store, &iter, parent, 0,
-				FILEVIEW_COLUMN_ICON, icon,
-				FILEVIEW_COLUMN_NAME, path_arr[level],
-				FILEVIEW_COLUMN_COLOR, project ? NULL : &s_external_color, -1);
-		}
-		else if (patterns_match(source_patterns, path_arr[level]))
-		{
-			if (! icon)
-				icon = g_icon_new_for_string("prjorg-source", NULL);
-
-			gtk_tree_store_insert_with_values(s_file_store, &iter, parent, 0,
-				FILEVIEW_COLUMN_ICON, icon,
-				FILEVIEW_COLUMN_NAME, path_arr[level],
-				FILEVIEW_COLUMN_COLOR, project ? NULL : &s_external_color, -1);
-		}
-		else
-		{
-			if (! icon)
-				icon = g_icon_new_for_string("prjorg-file", NULL);
-
-			gtk_tree_store_insert_with_values(s_file_store, &iter, parent, 0,
-				FILEVIEW_COLUMN_ICON, icon,
-				FILEVIEW_COLUMN_NAME, path_arr[level],
-				FILEVIEW_COLUMN_COLOR, project ? NULL : &s_external_color, -1);
-		}
+		gtk_tree_store_insert_with_values(s_file_store, &iter, parent, 0,
+			FILEVIEW_COLUMN_ICON, icon,
+			FILEVIEW_COLUMN_NAME, path_arr[level],
+			FILEVIEW_COLUMN_COLOR, project ? NULL : &s_external_color, -1);
 
 		if (icon)
 			g_object_unref(icon);
@@ -955,7 +939,7 @@ static void create_branch(gint level, GSList *leaf_list, GtkTreeIter *parent,
 		GtkTreeIter iter;
 		gchar **path_arr = dir_list->data;
 		gchar *last_dir_name;
-		GIcon *icon_dir = g_icon_new_for_string("folder", NULL);
+		GIcon *icon_dir = g_themed_icon_new("folder");
 
 		last_dir_name = path_arr[level];
 
@@ -1023,7 +1007,7 @@ static void load_project_root(PrjOrgRoot *root, GtkTreeIter *parent, GSList *hea
 {
 	GSList *lst = NULL;
 	GSList *path_list = NULL;
-	GSList *elem;
+	GSList *elem = NULL;
 	GHashTableIter iter;
 	gpointer key, value;
 
@@ -1070,7 +1054,7 @@ static void load_project_root(PrjOrgRoot *root, GtkTreeIter *parent, GSList *hea
 
 static void load_project(void)
 {
-	GSList *elem, *header_patterns, *source_patterns;
+	GSList *elem = NULL, *header_patterns, *source_patterns;
 	GtkTreeIter iter;
 	gboolean first = TRUE;
 	GIcon *icon_dir;
@@ -1080,7 +1064,7 @@ static void load_project(void)
 	if (!prj_org || !geany_data->app->project)
 		return;
 
-	icon_dir = g_icon_new_for_string("folder", NULL);
+	icon_dir = g_themed_icon_new("folder");
 
 	header_patterns = get_precompiled_patterns(prj_org->header_patterns);
 	source_patterns = get_precompiled_patterns(prj_org->source_patterns);
@@ -1139,8 +1123,7 @@ static gboolean find_in_tree(GtkTreeIter *parent, gchar **path_split, gint level
 		g_free(name);
 		if (cmpres == 0)
 		{
-			GtkTreeIter foo;
-			if (path_split[level+1] == NULL && !gtk_tree_model_iter_children (model, &foo, &iter))
+			if (path_split[level+1] == NULL)
 			{
 				*ret = iter;
 				return TRUE;
@@ -1156,27 +1139,21 @@ static gboolean find_in_tree(GtkTreeIter *parent, gchar **path_split, gint level
 }
 
 
-static gboolean follow_editor_on_idle(gpointer foo)
+static gboolean expand_path(gchar *utf8_expanded_path, gboolean select)
 {
 	GtkTreeIter root_iter, found_iter;
 	gchar *utf8_path = NULL;
 	gchar **path_split;
-	GeanyDocument *doc;
-	GSList *elem;
+	GSList *elem = NULL;
 	GtkTreeModel *model;
 
-	doc = document_get_current();
-
-	if (!doc || !doc->file_name || !geany_data->app->project || !prj_org)
-		return FALSE;
-
 	model = GTK_TREE_MODEL(s_file_store);
 	gtk_tree_model_iter_children(model, &root_iter, NULL);
 	foreach_slist (elem, prj_org->roots)
 	{
 		PrjOrgRoot *root = elem->data;
 
-		utf8_path = get_relative_path(root->base_dir, doc->file_name);
+		utf8_path = get_relative_path(root->base_dir, utf8_expanded_path);
 		if (utf8_path)
 			break;
 
@@ -1197,14 +1174,17 @@ static gboolean follow_editor_on_idle(gpointer foo)
 		GtkTreeSelection *treesel;
 
 		tree_path = gtk_tree_model_get_path (model, &found_iter);
-
 		gtk_tree_view_expand_to_path(GTK_TREE_VIEW(s_file_view), tree_path);
-		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(s_file_view), tree_path,
-			NULL, FALSE, 0.0, 0.0);
 
-		treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(s_file_view));
-		gtk_tree_selection_select_iter(treesel, &found_iter);
-		gtk_tree_path_free(tree_path);
+		if (select)
+		{
+			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(s_file_view), tree_path,
+				NULL, FALSE, 0.0, 0.0);
+
+			treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(s_file_view));
+			gtk_tree_selection_select_iter(treesel, &found_iter);
+			gtk_tree_path_free(tree_path);
+		}
 	}
 
 	g_free(utf8_path);
@@ -1214,19 +1194,99 @@ static gboolean follow_editor_on_idle(gpointer foo)
 }
 
 
+static gboolean expand_on_idle(ExpandData *expand_data)
+{
+	GeanyDocument *doc = document_get_current();
+
+	if (!prj_org)
+		return FALSE;
+
+	if (geany_data->app->project == expand_data->project &&
+		expand_data->expanded_paths)
+	{
+		gchar *item;
+		guint i;
+
+		foreach_ptr_array(item, i, expand_data->expanded_paths)
+			expand_path(item, FALSE);
+		g_ptr_array_free(expand_data->expanded_paths, TRUE);
+	}
+
+	g_free(expand_data);
+
+	if (!s_follow_editor || !doc || !doc->file_name)
+		return FALSE;
+
+	expand_path(doc->file_name, TRUE);
+
+	return FALSE;
+}
+
+
+static void on_map_expanded(GtkTreeView *tree_view, GtkTreePath *tree_path, GPtrArray *path_arr)
+{
+	GtkTreeIter iter;
+
+	if (gtk_tree_model_get_iter(GTK_TREE_MODEL(s_file_store), &iter, tree_path))
+	{
+		gchar *utf8_path = build_path(&iter);
+		gboolean replaced = FALSE;
+
+		if (path_arr->len > 0)
+		{
+			gchar *previous = path_arr->pdata[path_arr->len-1];
+			gchar *rel_path = get_relative_path(previous, utf8_path);
+
+			/* Filter-out redundant parent paths. gtk_tree_view_map_expanded_rows()
+			 * returns first parent paths and then nested paths so we can just
+			 * check if the last stored path is a parent of current path and
+			 * if so, replace it with curent path. */
+			if (rel_path)
+			{
+				g_free(previous);
+				path_arr->pdata[path_arr->len-1] = utf8_path;
+				replaced = TRUE;
+			}
+
+			g_free(rel_path);
+		}
+
+		if (!replaced)
+			g_ptr_array_add(path_arr, utf8_path);
+	}
+}
+
+
+static GPtrArray *get_expanded_paths(void)
+{
+	GPtrArray *expanded_paths = g_ptr_array_new_with_free_func(g_free);
+
+	gtk_tree_view_map_expanded_rows(GTK_TREE_VIEW(s_file_view),
+		(GtkTreeViewMappingFunc)on_map_expanded, expanded_paths);
+
+	return expanded_paths;
+}
+
+
 void prjorg_sidebar_update(gboolean reload)
 {
+	ExpandData *expand_data = g_new0(ExpandData, 1);
+
+	expand_data->project = geany_data->app->project;
+
 	if (reload)
 	{
+		expand_data->expanded_paths = get_expanded_paths();
+
 		load_project();
 		/* we get color information only after the sidebar is realized -
 		 * perform reload later if this is not the case yet */
 		if (!gtk_widget_get_realized(s_toolbar))
 			s_pending_reload = TRUE;
 	}
-	if (s_follow_editor)
-		/* perform on idle - avoids unnecessary jumps on project load */
-		plugin_idle_add(geany_plugin, (GSourceFunc)follow_editor_on_idle, NULL);
+
+	/* perform on idle - avoids unnecessary jumps on project load */
+	plugin_idle_add(geany_plugin, (GSourceFunc)expand_on_idle, expand_data);
 }
 
 
@@ -1267,8 +1327,8 @@ void prjorg_sidebar_init(void)
 
 	g_signal_connect (s_toolbar, "realize", G_CALLBACK (sidebar_realized_cb), NULL);
 
-	item = GTK_WIDGET(gtk_tool_button_new(NULL, NULL));
-	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON(item), "prjorg-refresh");
+	image = gtk_image_new_from_icon_name("view-refresh", GTK_ICON_SIZE_SMALL_TOOLBAR);
+	item = GTK_WIDGET(gtk_tool_button_new(image, NULL));
 	gtk_widget_set_tooltip_text(item, _("Reload all"));
 	g_signal_connect(item, "clicked", G_CALLBACK(on_reload_project), NULL);
 	gtk_container_add(GTK_CONTAINER(s_toolbar), item);
@@ -1276,8 +1336,8 @@ void prjorg_sidebar_init(void)
 	item = GTK_WIDGET(gtk_separator_tool_item_new());
 	gtk_container_add(GTK_CONTAINER(s_toolbar), item);
 
-	item = GTK_WIDGET(gtk_tool_button_new(NULL, NULL));
-	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON(item), "prjorg-add-external");
+	image = gtk_image_new_from_icon_name("folder-new", GTK_ICON_SIZE_SMALL_TOOLBAR);
+	item = GTK_WIDGET(gtk_tool_button_new(image, NULL));
 	gtk_widget_set_tooltip_text(item, _("Add external directory"));
 	g_signal_connect(item, "clicked", G_CALLBACK(on_add_external), NULL);
 	gtk_container_add(GTK_CONTAINER(s_toolbar), item);
@@ -1286,15 +1346,15 @@ void prjorg_sidebar_init(void)
 	item = GTK_WIDGET(gtk_separator_tool_item_new());
 	gtk_container_add(GTK_CONTAINER(s_toolbar), item);
 
-	item = GTK_WIDGET(gtk_tool_button_new(NULL, NULL));
-	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON(item), "prjorg-expand");
+	image = gtk_image_new_from_icon_name("list-add", GTK_ICON_SIZE_SMALL_TOOLBAR);
+	item = GTK_WIDGET(gtk_tool_button_new(image, NULL));
 	gtk_widget_set_tooltip_text(item, _("Expand all"));
 	g_signal_connect(item, "clicked", G_CALLBACK(on_expand_all), NULL);
 	gtk_container_add(GTK_CONTAINER(s_toolbar), item);
 	s_project_toolbar.expand = item;
 
-	item = GTK_WIDGET(gtk_tool_button_new(NULL, NULL));
-	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON(item), "prjorg-collapse");
+	image = gtk_image_new_from_icon_name("list-remove", GTK_ICON_SIZE_SMALL_TOOLBAR);
+	item = GTK_WIDGET(gtk_tool_button_new(image, NULL));
 	gtk_widget_set_tooltip_text(item, _("Collapse to project root"));
 	g_signal_connect(item, "clicked", G_CALLBACK(on_collapse_all), NULL);
 	gtk_container_add(GTK_CONTAINER(s_toolbar), item);
@@ -1304,8 +1364,8 @@ void prjorg_sidebar_init(void)
 	gtk_container_add(GTK_CONTAINER(s_toolbar), item);
 
 	item = GTK_WIDGET(gtk_toggle_tool_button_new());
+	gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(item), "go-jump");
 	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(item), TRUE);
-	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON(item), "prjorg-follow");
 	gtk_widget_set_tooltip_text(item, _("Follow active editor"));
 	g_signal_connect(item, "clicked", G_CALLBACK(on_follow_active), NULL);
 	gtk_container_add(GTK_CONTAINER(s_toolbar), item);
@@ -1359,7 +1419,7 @@ void prjorg_sidebar_init(void)
 
 	s_popup_menu.widget = gtk_menu_new();
 
-	image = gtk_image_new_from_icon_name("prjorg-expand", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name("list-add", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	item = gtk_image_menu_item_new_with_mnemonic(_("Expand All"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
