@@ -407,11 +407,13 @@ gboolean dialogs_directory_settings(WB_PROJECT_DIR *directory)
 gboolean dialogs_workbench_settings(WORKBENCH *workbench)
 {
 	gint result;
-	GtkWidget *w_rescan_projects_on_open;
+	GtkWidget *w_rescan_projects_on_open, *w_enable_live_update, *w_expand_on_hover;
 	GtkWidget *dialog, *content_area;
 	GtkWidget *vbox, *hbox, *table;
 	GtkDialogFlags flags;
 	gboolean changed, rescan_projects_on_open, rescan_projects_on_open_old;
+	gboolean enable_live_update, enable_live_update_old;
+	gboolean expand_on_hover, expand_on_hover_old;
 
 	/* Create the widgets */
 	flags = GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT;
@@ -432,11 +434,28 @@ gboolean dialogs_workbench_settings(WORKBENCH *workbench)
 	w_rescan_projects_on_open = gtk_check_button_new_with_mnemonic(_("_Rescan all projects on open"));
 	ui_table_add_row(GTK_TABLE(table), 0, w_rescan_projects_on_open, NULL);
 	gtk_widget_set_tooltip_text(w_rescan_projects_on_open,
-		_("If the option is activated (default), then all projects will be re-scanned"
-		  " on opening of the workbench."));
+		_("If the option is activated (default), then all projects will be re-scanned "
+		  "on opening of the workbench."));
 	rescan_projects_on_open_old = workbench_get_rescan_projects_on_open(workbench);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w_rescan_projects_on_open), rescan_projects_on_open_old);
 
+	w_enable_live_update = gtk_check_button_new_with_mnemonic(_("_Enable live update"));
+	ui_table_add_row(GTK_TABLE(table), 1, w_enable_live_update, NULL);
+	gtk_widget_set_tooltip_text(w_enable_live_update,
+		_("If the option is activated (default), then the list of files and the sidebar "
+		  "will be updated automatically if a file or directory is created, removed or renamed. "
+		  "A manual re-scan is not required if the option is enabled."));
+	enable_live_update_old = workbench_get_enable_live_update(workbench);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w_enable_live_update), enable_live_update_old);
+
+	w_expand_on_hover = gtk_check_button_new_with_mnemonic(_("_Expand on hover"));
+	ui_table_add_row(GTK_TABLE(table), 2, w_expand_on_hover, NULL);
+	gtk_widget_set_tooltip_text(w_expand_on_hover,
+		_("If the option is activated, then a tree node in the sidebar "
+		  "will be expanded or collapsed by hovering over it with the mouse cursor."));
+	expand_on_hover_old = workbench_get_expand_on_hover(workbench);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w_expand_on_hover), expand_on_hover_old);
+
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 6);
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -456,6 +475,18 @@ gboolean dialogs_workbench_settings(WORKBENCH *workbench)
 			changed = TRUE;
 			workbench_set_rescan_projects_on_open(workbench, rescan_projects_on_open);
 		}
+		enable_live_update = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w_enable_live_update));
+		if (enable_live_update != enable_live_update_old)
+		{
+			changed = TRUE;
+			workbench_set_enable_live_update(workbench, enable_live_update);
+		}
+		expand_on_hover = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w_expand_on_hover));
+		if (expand_on_hover != expand_on_hover_old)
+		{
+			changed = TRUE;
+			workbench_set_expand_on_hover(workbench, expand_on_hover);
+		}
 	}
 
 	gtk_widget_destroy(dialog);
