@@ -29,6 +29,7 @@
 #include <../../utils/src/filelist.h>
 #include "wb_globals.h"
 #include "wb_project.h"
+#include "sidebar.h"
 #include "utils.h"
 
 extern GeanyData *geany_data;
@@ -40,6 +41,13 @@ typedef enum
 	WB_PROJECT_TAG_PREFS_NO,
 }WB_PROJECT_TAG_PREFS;
 
+typedef struct
+{
+	WB_PROJECT_IDLE_ACTION_ID id;
+	gpointer param_a;
+	gpointer param_b;
+}WB_PROJECT_IDLE_ACTION;
+
 typedef struct
 {
 	GKeyFile *kf;
@@ -64,13 +72,21 @@ struct S_WB_PROJECT
 	gchar     *filename;
 	gchar     *name;
 	gboolean  modified;
-	GSList    *s_idle_add_funcs;
-	GSList    *s_idle_remove_funcs;
+	//GSList    *s_idle_add_funcs;
+	//GSList    *s_idle_remove_funcs;
 	GSList    *directories;  /* list of WB_PROJECT_DIR; */
 	WB_PROJECT_TAG_PREFS generate_tag_prefs;
 	GPtrArray *bookmarks;
 };
 
+typedef struct
+{
+	guint len;
+	const gchar *string;
+}WB_PROJECT_TEMP_DATA;
+
+static GSList *s_idle_actions = NULL;
+static void wb_project_dir_update_tags(WB_PROJECT_DIR *root);
 
 /** Set the projects modified marker.
  *
@@ -212,26 +228,26 @@ static gboolean match_basename(gconstpointer pft, gconstpointer user_data)
 
 
 /* Clear idle queue */
-static void wb_project_clear_idle_queue(GSList **queue)
+static void wb_project_clear_idle_queue(void)
 {
-	if (queue == NULL || *queue == NULL)
+	if (s_idle_actions == NULL)
 	{
 		return;
 	}
 
-	g_slist_free_full(*queue, g_free);
-	*queue = NULL;
+	g_slist_free_full(s_idle_actions, g_free);
+	s_idle_actions = NULL;
 }
 
 
 /* Create a new project dir with base path "utf8_base_dir" */
-static WB_PROJECT_DIR *wb_project_dir_new(const gchar *utf8_base_dir)
+static WB_PROJECT_DIR *wb_project_dir_new(WB_PROJECT *prj, const gchar *utf8_base_dir)
 {
 	guint offset;
 
 	if (utf8_base_dir == NULL)
 	{
-	    return NULL;
+		return NULL;
 	}
 	WB_PROJECT_DIR *dir = g_new0(WB_PROJECT_DIR, 1);
 	dir->base_dir = g_strdup(utf8_base_dir);
@@ -250,6 +266,7 @@ static WB_PROJECT_DIR *wb_project_dir_new(const gchar *utf8_base_dir)
 	}
 	dir->name = g_strdup(&(dir->base_dir[offset]));
 	dir->is_prj_base_dir = FALSE;
+
 	return dir;
 }
 
@@ -551,6 +568,201 @@ static guint wb_project_dir_rescan_int(WB_PROJECT *prj, WB_PROJECT_DIR *root)
 }
 
 
+/* Add a new file to the project directory and update the sidebar. */
+static void wb_project_dir_add_file_int(WB_PROJECT *prj, WB_PROJECT_DIR *root, const gchar *filepath)
+{
+	gboolean matches;
+	gchar **file_patterns = NULL;
+	SIDEBAR_CONTEXT context;
+	WB_MONITOR *monitor = NULL;
+
+	if (root->file_patterns && root->file_patterns[0])
+	{
+		file_patterns = root->file_patterns;
+	}
+
+	matches = gp_filelist_filepath_matches_patterns(filepath,
+		file_patterns, root->ignored_dirs_patterns, root->ignored_file_patterns);
+	if (!matches)
+	{
+		/* Ignore it. */
+		return;
+	}
+
+	/* Update file table and counters. */
+	g_hash_table_insert(root->file_table, g_strdup(filepath), NULL);
+	if (g_file_test(filepath, G_FILE_TEST_IS_DIR))
+	{
+		root->subdir_count++;
+		monitor = workbench_get_monitor(wb_globals.opened_wb);
+		wb_monitor_add_dir(monitor, prj, root, filepath);
+	}
+	else if (g_file_test(filepath, G_FILE_TEST_IS_REGULAR))
+	{
+		root->file_count++;
+	}
+
+	/* Update sidebar. */
+	memset(&context, 0, sizeof(context));
+	context.project = prj;
+	context.directory = root;
+	context.file = (gchar *)filepath;
+	sidebar_update(SIDEBAR_CONTEXT_FILE_ADDED, &context);
+
+	/* If the file is a directory we also have to manually add all files
+	   contained in it. */
+	if (monitor != NULL)
+	{
+		GSList *scanned, *elem = NULL;
+
+		scanned = gp_filelist_scan_directory_full(&(root->file_count), &(root->subdir_count),
+			filepath, file_patterns, root->ignored_dirs_patterns, root->ignored_file_patterns,
+			FILELIST_FLAG_ADD_DIRS);
+
+		foreach_slist(elem, scanned)
+		{
+			char *path = elem->data;
+
+			if (path)
+			{
+				wb_project_dir_add_file(prj, root, path);
+			}
+		}
+
+		g_slist_foreach(scanned, (GFunc) g_free, NULL);
+		g_slist_free(scanned);
+	}
+}
+
+
+/** Add a new file to the project directory and update the sidebar.
+ * 
+ * The file is only added if it matches the pattern settings.
+ *
+ * @param prj      The project to add it to.
+ * @param root     The directory to add it to.
+ * @param filepath The file to add.
+ *
+ **/
+void wb_project_dir_add_file(WB_PROJECT *prj, WB_PROJECT_DIR *root, const gchar *filepath)
+{
+	wb_project_dir_add_file_int(prj, root, filepath);
+	wb_project_add_idle_action(WB_PROJECT_IDLE_ACTION_ID_UPDATE_TAGS,
+		root, NULL);
+
+}
+
+
+/* Check if the filepath is equal for the length of the directory path in px_temp */
+static gboolean wb_project_dir_remove_child (gpointer key, gpointer value, gpointer user_data)
+{
+	WB_PROJECT_TEMP_DATA *px_temp;
+	TMSourceFile *sf;
+
+	px_temp = user_data;
+	if (strncmp(px_temp->string, key, px_temp->len) == 0)
+	{
+		/* We found a child of our removed directory.
+		   Remove it from the hash table. This will also free
+		   the tags. We do not need to update the sidebar as we
+		   already deleted the parent directory/node. */
+		sf = value;
+		if (sf != NULL)
+		{
+			tm_workspace_remove_source_file(sf);
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
+/** Remove a file from the project directory and update the sidebar.
+ * 
+ * If the file still exists, it is only removed if it matches the pattern settings.
+ *
+ * @param prj      The project to remove it from.
+ * @param root     The directory to remove it from.
+ * @param filepath The file to remove.
+ *
+ **/
+void wb_project_dir_remove_file(WB_PROJECT *prj, WB_PROJECT_DIR *root, const gchar *filepath)
+{
+	gboolean matches, was_dir;
+	gchar **file_patterns = NULL;
+	WB_MONITOR *monitor;
+
+	if (root->file_patterns && root->file_patterns[0])
+	{
+		file_patterns = root->file_patterns;
+	}
+
+	if (g_file_test(filepath, G_FILE_TEST_EXISTS))
+	{
+		matches = gp_filelist_filepath_matches_patterns(filepath,
+			file_patterns, root->ignored_dirs_patterns, root->ignored_file_patterns);
+	}
+	else
+	{
+		/* If the file does not exist any more, then always try to remove it. */
+		matches = TRUE;
+	}
+
+	if (matches)
+	{
+		SIDEBAR_CONTEXT context;
+		TMSourceFile *sf;
+
+		/* Update file table and counters. */
+		sf = g_hash_table_lookup (root->file_table, filepath);
+		if (sf != NULL)
+		{
+			tm_workspace_remove_source_file(sf);
+		}
+		g_hash_table_remove(root->file_table, filepath);
+
+		/* If the file already has been deleted, we cannot determine if it
+		   was a file or directory at this point. But the monitors will
+		   help us out, see code at end of function. */
+
+		/* Update sidebar. */
+		memset(&context, 0, sizeof(context));
+		context.project = prj;
+		context.directory = root;
+		context.file = (gchar *)filepath;
+		sidebar_update(SIDEBAR_CONTEXT_FILE_REMOVED, &context);
+	}
+
+	/* Remove the file monitor for filepath. This will only return TRUE
+	   if there is a file monitor for filepath and that means that file-
+	   path was a directory. So we can determine if filepath was a dir
+	   or not even if it has been deleted. */
+	monitor = workbench_get_monitor(wb_globals.opened_wb);
+	was_dir = wb_monitor_remove_dir(monitor, filepath);
+	if (was_dir)
+	{
+		WB_PROJECT_TEMP_DATA x_temp;
+
+		x_temp.len = strlen(filepath);
+		x_temp.string = filepath;
+		g_hash_table_foreach_remove(root->file_table,
+			wb_project_dir_remove_child, &x_temp);
+
+		if (root->subdir_count > 0)
+		{
+			root->subdir_count--;
+		}
+	}
+	else
+	{
+		if (root->file_count > 0)
+		{
+			root->file_count--;
+		}
+	}
+}
+
+
 /* Stolen and modified version from Geany. The only difference is that Geany
  * first looks at shebang inside the file and then, if it fails, checks the
  * file extension. Opening every file is too expensive so instead check just
@@ -613,15 +825,22 @@ static void wb_project_dir_regenerate_tags(WB_PROJECT_DIR *root, G_GNUC_UNUSED g
 	while (g_hash_table_iter_next(&iter, &key, &value))
 	{
 		TMSourceFile *sf;
-		gchar *utf8_path = key;
-		gchar *locale_path = utils_get_locale_from_utf8(utf8_path);
 
-		sf = tm_source_file_new(locale_path, filetypes_detect(utf8_path)->name);
-		if (sf && !document_find_by_filename(utf8_path))
-			g_ptr_array_add(source_files, sf);
+		sf = NULL;
+		if (g_file_test(key, G_FILE_TEST_IS_REGULAR))
+		{
+			gchar *utf8_path = key;
+			gchar *locale_path = utils_get_locale_from_utf8(utf8_path);
 
-		g_hash_table_insert(file_table, g_strdup(utf8_path), sf);
-		g_free(locale_path);
+			sf = tm_source_file_new(locale_path, filetypes_detect(utf8_path)->name);
+			if (sf && !document_find_by_filename(utf8_path))
+				g_ptr_array_add(source_files, sf);
+
+			g_free(locale_path);
+		}
+
+		/* Add all files to the file-table (files and dirs)! */
+		g_hash_table_insert(file_table, g_strdup(key), sf);
 	}
 	g_hash_table_destroy(root->file_table);
 	root->file_table = file_table;
@@ -631,6 +850,37 @@ static void wb_project_dir_regenerate_tags(WB_PROJECT_DIR *root, G_GNUC_UNUSED g
 }
 
 
+/* Update tags for new files */
+static void wb_project_dir_update_tags(WB_PROJECT_DIR *root)
+{
+	GHashTableIter iter;
+	gpointer key, value;
+	GPtrArray *source_files;
+
+	source_files = g_ptr_array_new();
+	g_hash_table_iter_init(&iter, root->file_table);
+	while (g_hash_table_iter_next(&iter, &key, &value))
+	{
+		if (value == NULL)
+		{
+			TMSourceFile *sf;
+			gchar *utf8_path = key;
+			gchar *locale_path = utils_get_locale_from_utf8(utf8_path);
+
+			sf = tm_source_file_new(locale_path, filetypes_detect(utf8_path)->name);
+			if (sf && !document_find_by_filename(utf8_path))
+				g_ptr_array_add(source_files, sf);
+
+			g_hash_table_insert(root->file_table, g_strdup(utf8_path), sf);
+			g_free(locale_path);
+		}
+	}
+
+	tm_workspace_add_source_files(source_files);
+	g_ptr_array_free(source_files, TRUE);
+}
+
+
 /** Rescan/update the file list of a project dir.
  *
  * @param project   The project to which directory belongs
@@ -661,14 +911,14 @@ void wb_project_rescan(WB_PROJECT *prj)
 {
 	GSList *elem = NULL;
 	guint filenum = 0;
+	GHashTableIter iter;
 
 	if (!prj)
 	{
 		return;
 	}
 
-	wb_project_clear_idle_queue(&prj->s_idle_add_funcs);
-	wb_project_clear_idle_queue(&prj->s_idle_remove_funcs);
+	wb_project_clear_idle_queue();
 
 	foreach_slist(elem, prj->directories)
 	{
@@ -679,6 +929,37 @@ void wb_project_rescan(WB_PROJECT *prj)
 	{
 		g_slist_foreach(prj->directories, (GFunc)wb_project_dir_regenerate_tags, NULL);
 	}
+
+	/* Create file monitors for directories. */
+	if (workbench_get_enable_live_update(wb_globals.opened_wb) == TRUE)
+	{
+		WB_MONITOR *monitor;
+
+		monitor = workbench_get_monitor(wb_globals.opened_wb);
+		foreach_slist(elem, prj->directories)
+		{
+			gpointer path, value;
+			GHashTable *filehash;
+			gchar *abs_path;
+
+			/* First add monitor for base dir */
+			abs_path = get_combined_path(wb_project_get_filename(prj),
+										wb_project_dir_get_base_dir(elem->data));
+			wb_monitor_add_dir(monitor, prj, elem->data, abs_path);
+			g_free(abs_path);
+
+			/* Now add all dirs in file table */
+			filehash = ((WB_PROJECT_DIR *)elem->data)->file_table;
+			g_hash_table_iter_init(&iter, filehash);
+			while (g_hash_table_iter_next (&iter, &path, &value))
+			{
+				if (path != NULL && g_file_test(path, G_FILE_TEST_IS_DIR))
+				{
+					wb_monitor_add_dir(monitor, prj, elem->data, path);
+				}
+			}
+		}
+	}
 }
 
 
@@ -732,95 +1013,22 @@ gboolean wb_project_file_is_included(WB_PROJECT *prj, const gchar *filename)
 }
 
 
-static gboolean add_tm_idle(gpointer foo)
+/* Add single tm file. Only to be called on-idle! */
+static void wb_project_add_single_tm_file(WB_PROJECT *prj, const gchar *filename)
 {
-	WB_PROJECT *prj;
-	GSList *elem2 = NULL;
+	GSList *elem = NULL;
 
-	prj = (WB_PROJECT *)foo;
-	if (prj == NULL || prj->s_idle_add_funcs == NULL)
+	foreach_slist (elem, prj->directories)
 	{
-		return FALSE;
-	}
+		WB_PROJECT_DIR *dir = elem->data;
+		TMSourceFile *sf = g_hash_table_lookup(dir->file_table, filename);
 
-	foreach_slist (elem2, prj->s_idle_add_funcs)
-	{
-		GSList *elem = NULL;
-		gchar *utf8_fname = elem2->data;
-
-		foreach_slist (elem, prj->directories)
+		if (sf != NULL && !document_find_by_filename(filename))
 		{
-			WB_PROJECT_DIR *dir = elem->data;
-			TMSourceFile *sf = g_hash_table_lookup(dir->file_table, utf8_fname);
-
-			if (sf != NULL && !document_find_by_filename(utf8_fname))
-			{
-				tm_workspace_add_source_file(sf);
-				break;  /* single file representation in TM is enough */
-			}
+			tm_workspace_add_source_file(sf);
+			break;  /* single file representation in TM is enough */
 		}
 	}
-
-	wb_project_clear_idle_queue(&(prj->s_idle_add_funcs));
-
-	return FALSE;
-}
-
-
-/* This function gets called when document is being closed by Geany and we need
- * to add the TMSourceFile from the tag manager because Geany removes it on
- * document close.
- *
- * Additional problem: The tag removal in Geany happens after this function is called.
- * To be sure, perform on idle after this happens (even though from my knowledge of TM
- * this shouldn't probably matter). */
-void wb_project_add_single_tm_file(WB_PROJECT *prj, const gchar *filename)
-{
-	if (prj == NULL)
-	{
-		return;
-	}
-
-	if (prj->s_idle_add_funcs == NULL)
-	{
-		plugin_idle_add(wb_globals.geany_plugin, (GSourceFunc)add_tm_idle, prj);
-	}
-
-	prj->s_idle_add_funcs = g_slist_prepend(prj->s_idle_add_funcs, g_strdup(filename));
-}
-
-
-static gboolean remove_tm_idle(gpointer foo)
-{
-	WB_PROJECT *prj;
-	GSList *elem2 = NULL;
-
-	prj = (WB_PROJECT *)foo;
-	if (prj == NULL || prj->s_idle_remove_funcs == NULL)
-	{
-		return FALSE;
-	}
-
-	foreach_slist (elem2, prj->s_idle_remove_funcs)
-	{
-
-		GSList *elem = NULL;
-		gchar *utf8_fname = elem2->data;
-
-		foreach_slist (elem, prj->directories)
-		{
-			WB_PROJECT_DIR *dir = elem->data;
-			TMSourceFile *sf = g_hash_table_lookup(dir->file_table, utf8_fname);
-
-			if (sf != NULL)
-			{
-				tm_workspace_remove_source_file(sf);
-			}
-		}
-	}
-
-	wb_project_clear_idle_queue(&(prj->s_idle_remove_funcs));
-	return FALSE;
 }
 
 
@@ -836,37 +1044,113 @@ static gboolean remove_tm_idle(gpointer foo)
  * when this function is called and if we remove the TmSourceFile now, line
  * number for the searched tag won't be found. For this reason delay the tag
  * TmSourceFile removal until idle */
-void wb_project_remove_single_tm_file(WB_PROJECT *prj, const gchar *utf8_filename)
+static void wb_project_remove_single_tm_file(WB_PROJECT *prj, const gchar *filename)
 {
-	if (prj == NULL)
+	GSList *elem = NULL;
+
+	foreach_slist (elem, prj->directories)
 	{
-		return;
+		WB_PROJECT_DIR *dir = elem->data;
+		TMSourceFile *sf = g_hash_table_lookup(dir->file_table, filename);
+
+		if (sf != NULL)
+		{
+			tm_workspace_remove_source_file(sf);
+		}
+	}
+}
+
+
+/* On-idle callback function. */
+static gboolean wb_project_on_idle_callback(gpointer foo)
+{
+	GSList *elem = NULL;
+	WB_PROJECT_IDLE_ACTION *action;
+
+	foreach_slist (elem, s_idle_actions)
+	{
+		action = elem->data;
+		switch (action->id)
+		{
+			case WB_PROJECT_IDLE_ACTION_ID_ADD_SINGLE_TM_FILE:
+				if (action->param_a != NULL && action->param_b != NULL)
+				{
+					wb_project_add_single_tm_file
+						(action->param_a, action->param_b);
+					g_free(action->param_b);
+				}
+			break;
+
+			case WB_PROJECT_IDLE_ACTION_ID_REMOVE_SINGLE_TM_FILE:
+				if (action->param_a != NULL && action->param_b != NULL)
+				{
+					wb_project_remove_single_tm_file
+						(action->param_a, action->param_b);
+					g_free(action->param_b);
+				}
+			break;
+
+			case WB_PROJECT_IDLE_ACTION_ID_UPDATE_TAGS:
+				if (action->param_a != NULL)
+				{
+					wb_project_dir_update_tags(action->param_a);
+				}
+			break;
+		}
 	}
 
-	if (prj->s_idle_remove_funcs == NULL)
+	wb_project_clear_idle_queue();
+
+	return FALSE;
+}
+
+
+/** Add a new idle action to the list.
+ *
+ * The function allocates a new WB_PROJECT_IDLE_ACTION structure and fills
+ * in the values passed. On-idle genay will then call wb_project_on_idle_callback
+ * and that function will call the function related to the action ID
+ * and pass the relevant parameters to it.
+ * 
+ * @param id The action to execute on-idle
+ * @param param_a Parameter A
+ * @param param_a Parameter B
+ *
+ **/
+void wb_project_add_idle_action(WB_PROJECT_IDLE_ACTION_ID id, gpointer param_a, gpointer param_b)
+{
+	WB_PROJECT_IDLE_ACTION *action;
+
+	action = g_new0(WB_PROJECT_IDLE_ACTION, 1);
+	action->id = id;
+	action->param_a = param_a;
+	action->param_b = param_b;
+
+	if (s_idle_actions == NULL)
 	{
-		plugin_idle_add(wb_globals.geany_plugin, (GSourceFunc)remove_tm_idle, prj);
+		plugin_idle_add(wb_globals.geany_plugin, (GSourceFunc)wb_project_on_idle_callback, NULL);
 	}
-	prj->s_idle_remove_funcs = g_slist_prepend(prj->s_idle_remove_funcs, g_strdup(utf8_filename));
+
+	s_idle_actions = g_slist_prepend(s_idle_actions, action);
 }
 
 
 /* Add a directory to the project */
 static WB_PROJECT_DIR *wb_project_add_directory_int(WB_PROJECT *prj, const gchar *dirname, gboolean rescan)
 {
-    if (prj != NULL)
-    {
-		WB_PROJECT_DIR *new_dir = wb_project_dir_new(dirname);
+	if (prj != NULL)
+	{
+		WB_PROJECT_DIR *new_dir = wb_project_dir_new(prj, dirname);
 
-        if (prj->directories != NULL)
-        {
+		if (prj->directories != NULL)
+		{
 			GSList *lst = prj->directories->next;
 			lst = g_slist_prepend(lst, new_dir);
 			lst = g_slist_sort(lst, (GCompareFunc)wb_project_dir_comparator);
 			prj->directories->next = lst;
-	    }
-	    else
-	    {
+		}
+		else
+		{
 			prj->directories = g_slist_append(prj->directories, new_dir);
 		}
 
@@ -875,8 +1159,8 @@ static WB_PROJECT_DIR *wb_project_add_directory_int(WB_PROJECT *prj, const gchar
 			wb_project_rescan(prj);
 		}
 		return new_dir;
-    }
-    return NULL;
+	}
+	return NULL;
 }
 
 
@@ -921,7 +1205,7 @@ gboolean wb_project_remove_directory (WB_PROJECT *prj, WB_PROJECT_DIR *dir)
 		wb_project_rescan(prj);
 		prj->modified = TRUE;
 	}
-    return FALSE;
+	return FALSE;
 }
 
 
@@ -1286,7 +1570,7 @@ gboolean wb_project_save(WB_PROJECT *prj, GError **error)
  * @return TRUE on success, FALSE otherwise
  *
  **/
-gboolean wb_project_load(WB_PROJECT *prj, gchar *filename, GError **error)
+gboolean wb_project_load(WB_PROJECT *prj, const gchar *filename, GError **error)
 {
 	GKeyFile *kf;
 	guint	 index;
@@ -1355,7 +1639,7 @@ gboolean wb_project_load(WB_PROJECT *prj, gchar *filename, GError **error)
 		gchar **bookmarks_strings;
 
 		/* Load project bookmarks from string list */
-		bookmarks_strings = g_key_file_get_string_list (kf, "Workbench", "Bookmarks", NULL, error);
+		bookmarks_strings = g_key_file_get_string_list (kf, "Workbench", "Bookmarks", NULL, NULL);
 		if (bookmarks_strings != NULL)
 		{
 			gchar **file, *abs_path;
@@ -1498,3 +1782,31 @@ void wb_project_free(WB_PROJECT *prj)
 	g_free(prj->name);
 	g_free(prj);
 }
+
+
+/** Check if dir is a valid reference to a directory of prj.
+ *
+ * @param prj The project to search in
+ * @param dir The directory to search for
+ * @return TRUE  dir is a directory in prj
+ *         FALSE dir was not found in prj
+ **/
+gboolean wb_project_is_valid_dir_reference(WB_PROJECT *prj, WB_PROJECT_DIR *dir)
+{
+	GSList *elem = NULL;
+
+	if (prj == NULL)
+	{
+		return FALSE;
+	}
+
+	foreach_slist(elem, prj->directories)
+	{
+		if (elem->data == dir)
+		{
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
